// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:ui';

import 'package:wyatt_ui_components/wyatt_wyatt_ui_components.dart';

class LoaderStyle extends ThemeStyle<LoaderStyle> {
  const LoaderStyle({
    this.colors,
    this.stroke,
  });

  /// Merges non-null `b` attributes in `a`
  static LoaderStyle? merge(
    LoaderStyle? a,
    LoaderStyle? b,
  ) {
    if (b == null) {
      return a?.copyWith();
    }
    if (a == null) {
      return b.copyWith();
    }

    return a.copyWith(
      colors: b.colors,
      stroke: b.stroke,
    );
  }

  /// Used for interpolation.
  static LoaderStyle? lerp(
    LoaderStyle? a,
    LoaderStyle? b,
    double t,
  ) {
    if (a == null || b == null) {
      return null;
    }
    // b.copyWith to return b attributes even if they are not lerped
    return b.copyWith(
      colors: MultiColor.lerp(a.colors, b.colors, t),
      stroke: lerpDouble(a.stroke, b.stroke, t),
    );
  }

  /// Gradient colors from start to end.
  final MultiColor? colors;

  /// Loader stroke width
  final double? stroke;

  @override
  LoaderStyle? mergeWith(LoaderStyle? other) => LoaderStyle.merge(this, other);

  @override
  LoaderStyle copyWith({
    MultiColor? colors,
    double? stroke,
  }) =>
      LoaderStyle(
        colors: colors ?? this.colors,
        stroke: stroke ?? this.stroke,
      );

  @override
  String toString() => 'LoaderStyle($colors, $stroke)';
}
