// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/widgets.dart';
import 'package:wyatt_ui_components/wyatt_wyatt_ui_components.dart';

class RichTextBuilderStyle extends ThemeStyle<RichTextBuilderStyle> {
  const RichTextBuilderStyle({
    this.defaultStyle,
    this.styles,
  });

  /// Merges non-null `b` attributes in `a`
  static RichTextBuilderStyle? merge(
    RichTextBuilderStyle? a,
    RichTextBuilderStyle? b,
  ) {
    if (b == null) {
      return a?.copyWith();
    }
    if (a == null) {
      return b.copyWith();
    }

    return a.copyWith(
      defaultStyle: b.defaultStyle,
      styles: b.styles,
    );
  }

  /// Used for interpolation.
  static RichTextBuilderStyle? lerp(
    RichTextBuilderStyle? a,
    RichTextBuilderStyle? b,
    double t,
  ) {
    if (a == null || b == null) {
      return null;
    }
    // b.copyWith to return b attributes even if they are not lerped
    return b.copyWith(
      defaultStyle: TextStyle.lerp(a.defaultStyle, b.defaultStyle, t),
      styles: b.styles, // TODO(wyatt): compute lerp value of each styles
    );
  }

  /// Default TextStyle used in this rich text component.
  final TextStyle? defaultStyle;

  /// Used styles in this rich text component.
  final Map<String, TextStyle>? styles;

  @override
  RichTextBuilderStyle? mergeWith(RichTextBuilderStyle? other) =>
      RichTextBuilderStyle.merge(this, other);

  @override
  RichTextBuilderStyle? copyWith({
    TextStyle? defaultStyle,
    Map<String, TextStyle>? styles,
  }) =>
      RichTextBuilderStyle(
        defaultStyle: defaultStyle ?? this.defaultStyle,
        styles: styles ?? this.styles,
      );
}
