// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/wyatt_wyatt_ui_components.dart';
import 'package:wyatt_ui_kit/wyatt_ui_kit.dart';

class LoaderThemeResolver
    extends ThemeResolver<LoaderStyle, LoaderThemeExtension, void> {
  const LoaderThemeResolver({
    required this.customStyleFn,
  });

  @override
  final LoaderStyle? Function(
    BuildContext context, {
    void extra,
  }) customStyleFn;

  /// Values taken from <https://api.flutter.dev/flutter/material/ElevatedButton/defaultStyleOf.html>
  @override
  LoaderStyle computeDefaultValue(
    BuildContext context, {
    void extra,
  }) =>
      LoaderStyle(
        colors: MultiColor([
          Theme.of(context).progressIndicatorTheme.color ??
              context.colorScheme.primary,
          context.colorScheme.onPrimary,
        ]),
        stroke: 4,
      );

  @override
  LoaderStyle? computeExtensionValueFn(
    BuildContext context,
    LoaderThemeExtension themeExtension, {
    void extra,
  }) =>
      LoaderStyle(
        colors: themeExtension.colors,
        stroke: themeExtension.stroke,
      );
}
