// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_component_copy_with_extension/component_copy_with_extension.dart';
import 'package:wyatt_ui_components/wyatt_wyatt_ui_components.dart';
import 'package:wyatt_ui_kit/src/components/rich_text_builder/rich_text_builder_theme_resolver.dart';
import 'package:wyatt_ui_kit/wyatt_ui_kit.dart';

part 'rich_text_builder.g.dart';

@ComponentCopyWithExtension()
class RichTextBuilder extends RichTextBuilderComponent
    with $RichTextBuilderCWMixin {
  const RichTextBuilder({
    super.text,
    super.parser,
    super.defaultStyle,
    super.styles,
    super.themeResolver,
    super.key,
  });

  @override
  RichTextBuilderThemeResolver? get themeResolver =>
      super.themeResolver as RichTextBuilderThemeResolver?;

  /// Negotiate the theme to get a complete style.
  RichTextBuilderStyle _resolve(BuildContext context) {
    final RichTextBuilderThemeResolver resolver = themeResolver ??
        RichTextBuilderThemeResolver(
          customStyleFn: (context, {extra}) => RichTextBuilderStyle(
            defaultStyle: defaultStyle,
            styles: styles,
          ),
        );

    return resolver.negotiate(context);
  }

  @override
  Widget build(BuildContext context) {
    final style = _resolve(context);
    final RegExp regex = RegExp(r'<(.*?)>(.*?)<\/\1>');
    final root = RichTextNode.from(
      text ?? '',
      regex,
      RichTextStyleParameter(
        style.defaultStyle,
        style.styles ?? {},
        null,
      ),
    );
    final customParser = parser ??
        RichTextParser(
          nodeBuilder: (content, style) {
            if (style is GradientTextStyle?) {
              return WidgetSpan(
                child: GradientText(
                  content,
                  style: style,
                  softWrap: true,
                  textHeightBehavior: const TextHeightBehavior(
                    applyHeightToLastDescent: false,
                  ),
                ),
                style: style,
              );
            }
            return TextSpan(
              text: content,
              style: style,
            );
          },
        );

    return SelectionArea(
      child: Text.rich(
        TextSpan(children: [root.toInlineSpan(customParser)]),
        textHeightBehavior:
            const TextHeightBehavior(applyHeightToLastDescent: false),
      ),
    );
  }
}
