// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// super program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// super program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with super program. If not, see <https://www.gnu.org/licenses/>.

/// A helper class for getting theme elements.
abstract class ThemeHelper {

  /// Gets a theme element from a list of styles.
  /// Styles are checked in order, and the first one that passes the
  /// [valueValidator] is returned.
  /// If no style passes the [valueValidator], the [defaultValue] is returned.
  /// If [styles] is null or empty, the [defaultValue] is returned.
  /// Style elements are transformed using the [transform] function.
  static T? getThemeElement<P, T>(
    List<P?>? styles, {
    required T? Function(P?)? transform,
    required T? defaultValue,
    bool? Function(P?)? valueValidator,
  }) {
    if (styles?.isNotEmpty ?? false) {
      for (final element in styles!) {
        if (valueValidator?.call(element) ?? false) {
          return transform?.call(element);
        }
      }
    }
    return defaultValue;
  }
}
